@@ -8,6 +8,7 @@ is_win32 = os.name == "nt"
 if is_py2:
     _str = str
     str = unicode
+    range = xrange
 
     def bytes(b=None, enc="ascii"):
         if b is None:
@@ -17,12 +18,10 @@ if is_py2:
         else:
             return _str(b)
 
-    from StringIO import StringIO as BytesIO
-
 elif is_py3:
     bytes = bytes
     str = str
-    from io import BytesIO
+    range = range
 
 
 try:
@@ -30,4 +29,5 @@ try:
 except ImportError:
     from .ordereddict import OrderedDict
 
-__all__ = ["is_py2", "is_py3", "is_win32", "str", "bytes", "BytesIO", "OrderedDict"]
+__all__ = ["is_py2", "is_py3", "is_win32", "str", "bytes", "range",
+           "OrderedDict"]
