@@ -1,6 +1,7 @@
 from livestreamer.compat import str
-from livestreamer.plugins import Plugin, PluginError, NoStreamsError
-from livestreamer.stream import RTMPStream, HLSStream
+from livestreamer.exceptions import PluginError, NoStreamsError
+from livestreamer.plugin import Plugin
+from livestreamer.stream import RTMPStream, HLSStream, HDSStream
 from livestreamer.utils import urlget, verifyjson, res_json
 
 import re
@@ -29,18 +30,27 @@ class SVTPlay(Plugin):
             if not ("url" in video and "playerType" in video):
                 continue
 
+            url = video["url"]
+
             if video["playerType"] == "flash":
-                if video["url"].startswith("rtmp"):
+                if url.startswith("rtmp"):
                     stream = RTMPStream(self.session, {
-                        "rtmp": video["url"],
+                        "rtmp": url,
                         "pageUrl": self.PageURL,
                         "swfVfy": self.SWFURL,
                         "live": True
                     })
                     streams[str(video["bitrate"]) + "k"] = stream
+                elif "manifest.f4m" in url:
+                    try:
+                        hdsstreams = HDSStream.parse_manifest(self.session, url)
+                        streams.update(hdsstreams)
+                    except IOError as err:
+                        self.logger.warning("Failed to get HDS manifest: {0}", err)
+
             elif video["playerType"] == "ios":
                 try:
-                    hlsstreams = HLSStream.parse_variant_playlist(self.session, video["url"])
+                    hlsstreams = HLSStream.parse_variant_playlist(self.session, url)
                     streams.update(hlsstreams)
                 except IOError as err:
                     self.logger.warning("Failed to get variant playlist: {0}", err)
