@@ -1,6 +1,7 @@
 from livestreamer.compat import str, bytes, urlparse
 from livestreamer.packages.flashmedia import AMF0Packet, AMFError
-from livestreamer.plugins import Plugin, PluginError, NoStreamsError
+from livestreamer.plugin import Plugin
+from livestreamer.exceptions import PluginError, NoStreamsError
 from livestreamer.stream import HLSStream, RTMPStream
 from livestreamer.utils import urlget
 
@@ -17,6 +18,10 @@ class UStreamTV(Plugin):
         return "ustream.tv" in url
 
     def _get_channel_id(self, url):
+        match = re.search("ustream.tv/embed/(\d+)", url)
+        if match:
+            return int(match.group(1))
+
         res = urlget(url)
 
         match = re.search("\"cid\":(\d+)", res.text)
@@ -85,8 +90,10 @@ class UStreamTV(Plugin):
                 if "streamVersionCdn" in info:
                     for name, cdn in info["streamVersionCdn"].items():
                         if "cdnStreamUrl" in cdn and "cdnStreamName" in cdn:
-                            streams["cdn_" + name] = self._create_stream(cdn["cdnStreamUrl"],
-                                                                         cdn["cdnStreamName"])
+                            cdnname = "live_alt_{0}".format(name)
+                            streams[cdnname] = self._create_stream(cdn["cdnStreamUrl"],
+                                                                   cdn["cdnStreamName"])
+
 
 
         return streams
