@@ -1,7 +1,8 @@
-from livestreamer.compat import str, bytes, parse_qs
-from livestreamer.plugins import Plugin, PluginError, NoStreamsError
-from livestreamer.stream import HTTPStream
-from livestreamer.utils import urlget, verifyjson, parse_json
+from livestreamer.compat import str, bytes
+from livestreamer.exceptions import PluginError, NoStreamsError
+from livestreamer.plugin import Plugin
+from livestreamer.stream import HTTPStream, HLSStream
+from livestreamer.utils import urlget, verifyjson, parse_json, parse_qsd
 
 import re
 import json
@@ -31,7 +32,7 @@ class Youtube(Plugin):
         streams = []
 
         for stream_qs in streammap.split(","):
-            stream = parse_qs(stream_qs)
+            stream = parse_qsd(stream_qs)
             streams.append(stream)
 
         return streams
@@ -72,16 +73,25 @@ class Youtube(Plugin):
             if not ("url" in streaminfo and "sig" in streaminfo):
                 continue
 
-            stream = HTTPStream(self.session, streaminfo["url"][0],
-                                params=dict(signature=streaminfo["sig"][0]))
+            stream = HTTPStream(self.session, streaminfo["url"],
+                                params=dict(signature=streaminfo["sig"]))
 
-            if streaminfo["itag"][0] in formatmap:
-                quality = formatmap[streaminfo["itag"][0]]
+            if streaminfo["itag"] in formatmap:
+                quality = formatmap[streaminfo["itag"]]
             else:
                 quality = streaminfo["quality"]
 
             streams[quality] = stream
 
+        if "hlsvp" in args:
+            url = args["hlsvp"]
+
+            try:
+                hlsstreams = HLSStream.parse_variant_playlist(self.session, url)
+                streams.update(hlsstreams)
+            except IOError as err:
+                self.logger.warning("Failed to get variant playlist: {0}", err)
+
         return streams
 
 __plugin__ = Youtube
