@@ -1,13 +1,13 @@
-#!/usr/bin/env python
-
-from . import Stream, StreamError
+from .stream import Stream
 from ..compat import str, bytes, urlparse
+from ..exceptions import StreamError
 from ..utils import Buffer, swfdecompress, swfverify, urlget, urlopen
 
 from ..packages.flashmedia import FLV, FLVError
 from ..packages.flashmedia.tag import ScriptData
 
 import base64
+import io
 import hashlib
 import hmac
 import random
@@ -46,7 +46,7 @@ def cache_bust_string(length):
 
     return rval
 
-class AkamaiHDStreamFD(Stream):
+class AkamaiHDStreamIO(io.IOBase):
     Version = "2.5.8"
     FlashVersion = "LNX 11,1,102,63"
 
@@ -71,10 +71,9 @@ class AkamaiHDStreamFD(Stream):
     }
 
     def __init__(self, session, url, swf=None, seek=None):
-        Stream.__init__(self, session)
-
         parsed = urlparse(url)
 
+        self.session = session
         self.logger = self.session.logger.new_module("stream.akamaihd")
         self.host = ("{scheme}://{netloc}").format(scheme=parsed.scheme, netloc=parsed.netloc)
         self.streamname = parsed.path[1:]
@@ -136,7 +135,10 @@ class AkamaiHDStreamFD(Stream):
         self.send_control("sendingNewToken", headers=headers,
                           swf=self.swf)
 
-    def send_control(self, cmd, headers={}, **params):
+    def send_control(self, cmd, headers=None, **params):
+        if not headers:
+            headers = {}
+
         url = self.ControlURLFormat.format(host=self.host,
                                            streamname=self.streamname)
 
@@ -147,7 +149,7 @@ class AkamaiHDStreamFD(Stream):
         return urlopen(url, headers=headers, params=params,
                        data=self.ControlData, exception=StreamError)
 
-    def read(self, size=0):
+    def read(self, size=-1):
         if not self.flv:
             return b""
 
@@ -223,6 +225,8 @@ class AkamaiHDStreamFD(Stream):
 
 
 class AkamaiHDStream(Stream):
+    __shortname__ = "akamaihd"
+
     def __init__(self, session, url, swf=None, seek=None):
         Stream.__init__(self, session)
 
@@ -230,8 +234,16 @@ class AkamaiHDStream(Stream):
         self.swf = swf
         self.url = url
 
+    def __repr__(self):
+        return ("<AkamaiHDStream({0!r}, "
+                "swf={1!r})>".format(self.url, self.swf))
+
+    def __json__(self):
+        return dict(type=AkamaiHDStream.shortname(),
+                    url=self.url, swf=self.swf)
+
     def open(self):
-        stream = AkamaiHDStreamFD(self.session, self.url,
+        stream = AkamaiHDStreamIO(self.session, self.url,
                                   self.swf, self.seek)
 
         return stream.open()
