@@ -1,35 +1,50 @@
-from . import StreamProcess, StreamError
-from ..compat import str, sh, pbs_compat
+from .streamprocess import StreamProcess
+from ..compat import str, urljoin
+from ..exceptions import StreamError
+from ..packages import pbs as sh
+from ..utils import rtmpparse
 
 from time import sleep
 
 import re
 
 class RTMPStream(StreamProcess):
+    __shortname__ = "rtmp"
+
     def __init__(self, session, params, redirect=False, timeout=30):
         StreamProcess.__init__(self, session, params, timeout)
 
         self.cmd = self.session.options.get("rtmpdump")
         self.redirect = redirect
         self.logger = session.logger.new_module("stream.rtmp")
-        self.params["flv"] = "-"
 
-        if self.session.options.get("rtmpdump-proxy"):
-            self.params["socks"] = self.session.options.get("rtmpdump-proxy")
+    def __repr__(self):
+        return ("<RTMPStream({0!r}, redirect={1!r}, "
+                "timeout={2!r})>").format(self.params, self.redirect,
+                                          self.timeout)
+
+    def __json__(self):
+        return dict(type=RTMPStream.shortname(),
+                    params=self.params)
 
     def open(self):
+        if self.session.options.get("rtmpdump-proxy"):
+            if not self._supports_param("socks"):
+                raise StreamError("Installed rtmpdump does not support --socks argument")
+
+            self.params["socks"] = self.session.options.get("rtmpdump-proxy")
+
         if "jtv" in self.params and not self._supports_param("jtv"):
             raise StreamError("Installed rtmpdump does not support --jtv argument")
 
-        if "socks" in self.params and not self._supports_param("socks"):
-            raise StreamError("Installed rtmpdump does not support --socks argument")
-
         if self.redirect:
             self._check_redirect()
 
+        self.params["flv"] = "-"
+
         return StreamProcess.open(self)
 
-    def _check_redirect(self, timeout=5):
+    def _check_redirect(self, timeout=20):
         cmd = self._check_cmd()
 
         params = self.params.copy()
@@ -44,10 +59,7 @@ class RTMPStream(StreamProcess):
         process_alive = True
 
         while elapsed < timeout and process_alive:
-            if pbs_compat:
-                process_alive = stream.process.returncode is None
-            else:
-                process_alive = stream.process.alive
+            process_alive = stream.process.returncode is None
 
             sleep(0.25)
             elapsed += 0.25
@@ -59,7 +71,7 @@ class RTMPStream(StreamProcess):
                 pass
 
         try:
-            stderr = stream.stderr
+            stderr = stream.stderr()
         except sh.ErrorReturnCode as err:
             self._update_redirect(err.stderr)
 
@@ -67,19 +79,17 @@ class RTMPStream(StreamProcess):
         tcurl, redirect = None, None
         stderr = str(stderr, "utf8")
 
-        m = re.search("DEBUG: tcUrl\s+: (.+)\n", stderr)
-        if m:
-            tcurl = m.group(1)
-
         m = re.search("DEBUG: Property: <Name:\s+redirect,\s+STRING:\s+(\w+://.+?)>", stderr)
         if m:
             redirect = m.group(1)
 
-        if tcurl and redirect:
+        if redirect:
             self.logger.debug("Found redirect tcUrl: {0}", redirect)
 
             if "rtmp" in self.params:
-                self.params["rtmp"] = self.params["rtmp"].replace(tcurl, redirect)
+                tcurl, playpath = rtmpparse(self.params["rtmp"])
+                rtmp = urljoin(redirect, playpath)
+                self.params["rtmp"] = rtmp
 
             if "tcUrl" in self.params:
                 self.params["tcUrl"] = redirect
